/* GCC */
#include <fstream>
#include <string>
#include <iomanip>
#include <cmath>
#include <vector>
#include <algorithm>

using namespace std;

long long gcd(long long a, long long b)
{
	return b == 0 ? a : gcd(b, a % b);
}

struct rational
{
	rational(long long n = 0, long long d = 1)
		: num(n), den(d)
	{}
	void norm(void)
	{
		if (den < 0)
			num = -num, den = -den;
		const long long g = gcd(abs(num), den);
		num /= g, den /= g;
	}
	long long num;
	long long den;
};

rational operator*(const rational& lhs, const rational& rhs)
{
	rational ret(lhs.num * rhs.num, lhs.den * rhs.den);
	ret.norm();
	return ret;
}

rational& operator/=(rational& lhs, const rational& rhs)
{
	lhs.num *= rhs.den, lhs.den *= rhs.num;
	lhs.norm();
	return lhs;
}

rational& operator-=(rational& lhs, const rational& rhs)
{
	lhs.num = lhs.num * rhs.den - lhs.den * rhs.num;
	lhs.den *= rhs.den;
	lhs.norm();
	return lhs;
}

int main(void)
{
	ios_base::sync_with_stdio(false);
	int n, m;
	ifstream in("linear.in");
	in >> n >> m;
	vector<vector<rational> > table(n, vector<rational>(m + 1));
	for (int i = 0; i < n; ++i)
	{
		for (int j = 0; j < m; ++j)
			in >> table[i][j].num;
		in >> table[i][m].num;
		table[i][m].num *= -1;
	}
	in.close();
	rational k;
	int row(0), col(0);
	bool inf = false;
	for (; col < m && row < n; ++col)
	{
		for (int i = row; i < n; ++i)
			if (table[i][col].num != 0)
			{
				swap(table[i], table[row]);
				break;
			}
		if (table[row][col].num == 0)
		{
			inf = true;
			continue;
		}
		k = table[row][col];
		for (int j = col; j <= m; ++j)
			table[row][j] /= k;
		for (int i = 0; i < n; ++i)
		{
			if (i == row)
				continue;
			k = table[i][col];
			for (int j = col; j <= m; ++j)
				table[i][j] -= k * table[row][j];
		}
		++row;
	}
	ofstream out("linear.out");
	for (int i = row; i < n; ++i)
		if (table[i][m].num != 0)
		{
			out << "No solution\n";
			out.close();
			return 0;
		}
    if(col < m)
        inf = true;
	if (inf)
		for (int i = 0; i < row; ++i)
		{
			for (int j = 0; j <= m; ++j)
			{
				if (j == m)
					table[i][j].num *= -1;
				out << ' ' << table[i][j].num;
				if (table[i][j].den != 1)
					out << '/' << table[i][j].den;
			}
			out << '\n';
		}
	else
	{
		for (int i = 0; i < m; ++i)
		{
			out << ' ' << -table[i][m].num;
			if (table[i][m].den != 1)
				out << '/' << table[i][m].den;
		}
		out << '\n';
	}
	out.close();
	return 0;
}
